"""Errors for PexelsAPI"""

class PexelsError(Exception):
    """
    Base exception for this module
    """
class QuotaExceedError(PexelsError):
    """Raises when total request limit for the monthly period ends"""
    pass

class APIError(PexelsError):
    """Raises when an error occurs on the API"""
    pass

class InvalidTokenError(PexelsError):
    """Raises when given API Token is invalid"""
    pass


"""
Client class for the Pexels Unofficial API Wrapper
Author: Joker Hacker
"""

from json import JSONDecodeError
import re
from typing import Any, Dict, Tuple, Optional, Union
import requests
from Pexels.constants import COLOR, LOCALE_SUPPORTED, ORIENTATION, SIZE
from Pexels.errors import PexelsError, QuotaExceedError
from Pexels.types import CollectionMediaResponse, CollectionResponse, Photo, PhotoResponse, Video, VideoResponse

class Client:
    """
    This object represents Client.

    .. code:: python

        client = Client(token="abcde12345")
    Note:
        * Whenever you are doing an API request make sure to show a prominent link to Pexels.
        You can use a text link (e.g. "Photos provided by Pexels") or a link with our logo.

        * Always credit our photographers when possible (e.g. "Photo by John Doe on Pexels" with a link to the photo page on Pexels).
    
        * Most Pexels API requests return multiple records at once. All of these endpoints are paginated, 
        and can return a maximum of 80 requests at one time. Each paginated request accepts the same parameters and returns the same pagination data in the response.

        * The `prev_page` and `next_page` response attributes will only be returned if there is a corresponding page. 

    Warning:
        * You may not copy or replicate core functionality of Pexels (including making Pexels content available as a wallpaper app).

        * Do not abuse the API. By default, the API is rate-limited to 200 requests per hour and 20,000 requests per month.

    Args:
        token (:obj:`str`): Unique authentication token.
        base_endpoint (:obj:`str`, optional): Base endpoint of the API,
            defaults to https://api.pexels.com/v1/
        video_endpoint (:obj:`str`, optional): Video endpoint of the API,
            defaults to https://api.pexels.com/videos
    """

    def __init__(
        self,
        token: str,
        base_endpoint: str = "https://api.pexels.com/v1/",
        video_endpoint: str = "https://api.pexels.com/videos"
    ):

        self._base_endpoint = base_endpoint
        self._video_endpoint = video_endpoint
        self._token = token
        self._header = {'Authorization': self._token}
        self.session = requests.Session()

    def _make_request(
        self,
        path: str,
        search_type: str,
        method: str = "get",
        query: Dict = {},
        **kwargs: Dict[Any, Any]
    ) -> Tuple[Union[Dict, str], requests.Response]:

        if search_type == 'photo':
            endpoint = self._base_endpoint
            req = self.session.request(
                    method, 
                    f'{endpoint}/{path}', 
                    headers=self._header, 
                    params=query,
                    **kwargs
                )
        elif search_type == 'video':
            endpoint = self._video_endpoint
            req = self.session.request(
                    method, 
                    f'{endpoint}/{path}', 
                    headers=self._header, 
                    params=query,
                    **kwargs,
                )
        else:
            raise PexelsError("Invalid parameter search_type given")

        if req.status_code in [200, 201]:
            try:
                return req.json(), req
            except JSONDecodeError:
                return req.text, req
        elif req.status_code == 400:
            raise PexelsError("Bad Request Caught")
        elif req.status_code == 429:
            raise QuotaExceedError("You have exceeded your rate limit.")
        else:
            raise PexelsError(f"{req.status_code} : {req.reason}")

    def search_photos(
        self, 
        query: str, 
        orientation: Optional[str] = "", 
        size: Optional[str] = "",
        color: Optional[str] = "",
        locale: Optional[str] = "", 
        page: Optional[int] = 1,
        per_page: Optional[int] = 15,
        **kwargs
    ) -> PhotoResponse:
        """
        This method enables you to search photos for any topic that you would like.
        For example your query could be something broad like `Nature`, `Tigers`, `People`.
        Or it could be something specific like Group of people working.
        
        Args:
            query (:obj:`str`): The search query. `Ocean`, `Tigers`, `Pears`, etc.
            orientation (:obj:`str`, optional): Desired photo orientation.
                list of supported orientations are available at :obj:`Pexels.constants.ORIENTATION`.
            size (:obj:`str`, optional): Minimum photo size.
                list of supported sizes are available at :obj:`Pexels.constants.SIZE`.
            color (:obj:`str`, optional): Desired photo color.
                list of supported colors are available at :obj:`Pexels.constants.COLOR`.
            locale (:obj:`str`, optional): The locale of the search you are performing.
                list of supported locales are available at :obj:`Pexels.constants.LOCALE_SUPPORTED`.
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.PhotoResponse`
        
        Raises:
            PexelsError: When invalid `orientation` or `color` or `size` or `locale` given or when `per_page` is above 80.
        """

        if orientation in ORIENTATION:
            pass
        elif orientation == "":
            pass
        else:
            raise PexelsError("Invalid value given for orientation, supported ones are landscape, portrait and square.")
        
        color_match = re.search(r'^#(?:[0-9a-fA-F]{3}){1,2}$', color)
        if color in COLOR:
            pass
        elif color_match:
            pass
        elif color == "":
            pass
        else:
            raise PexelsError("Invalid color name or hexadecimal code given.")

        if size in SIZE:
            pass
        elif size == "":
            pass
        else:
            raise PexelsError("Invalid photo size given, the supported ones are large, medium and small.")

        if locale == "":
            pass
        elif locale not in LOCALE_SUPPORTED:
            raise PexelsError("Invalid locale given.")

        if per_page > 80:
            raise PexelsError("per_page can not be more than 80")

        params = {
            'orientation': orientation, 
            'size': size, 
            'color': color, 
            'locale': locale, 
            'page': page,
            'per_page': per_page,
            **kwargs
        }

        data, req = self._make_request(f"search?query={query}", search_type='photo', query=params)
        return PhotoResponse(**data)

    def search_curated_photo(self, page: Optional[int] =1, per_page: Optional[int] = 15) -> PhotoResponse:
        """
        This method enables you to receive real-time photos curated by the Pexels team.
        We add at least one new photo per hour to our curated list so that you always get a changing selection of trending photos. 
        
        Args:
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.PhotoResponse`
        
        Raises:
            PexelsError: When `per_page` is above 80.
        """
        if per_page > 80:
            raise PexelsError("per_page can not be more than 80")

        params = {'page': page, 'per_page': per_page}

        data, req = self._make_request("curated", "photo", query=params)
        return PhotoResponse(**data)

    def get_photo(self, id: int) -> Photo:
        """
        Retrieve a specific `Photo` from its id.

        Args:
            id (:obj:`int`): The id of the photo you are requesting.
        
        Returns: 
            :class:`Pexels.types.Photo`
        """

        data, req = self._make_request(f"photos/{id}", "photo")
        return Photo(**data)

    def search_videos(
        self,
        query: str,
        orientation: str = "",
        size: str = "",
        locale:str = "",
        page: Optional[int] = 3,
        per_page: Optional[int] = 15,
        **kwargs
    ) -> VideoResponse:
        """
        This method enables you to search Videos for any topic that you would like.
        For example your query could be something broad like `Nature`, `Tigers`, `People`. Or it could be something specific like Group of people working. 
        
        Args:
            query (:obj:`str`): The search query. `Ocean`, `Tigers`, `Pears`, etc.
            orientation (:obj:`str`, optional): Desired Video orientation.
                list of supported orientations are available at :obj:`Pexels.constants.ORIENTATION`.
            size (:obj:`str`, optional): Minimum Video size.
                list of supported sizes are available at :obj:`Pexels.constants.SIZE`.
            locale (:obj:`str`, optional): The locale of the search you are performing.
                list of supported locales are available at :obj:`Pexels.constants.LOCALE_SUPPORTED`.
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.VideoResponse`
        
        Raises:
            PexelsError: When invalid `orientation` or `size` or `locale` given or when `per_page` is above 80.
        """

        if orientation in ORIENTATION:
            pass
        elif orientation == "":
            pass
        else:
            raise PexelsError("Invalid value given for orientation, supported ones are landscape, portrait and square.")
        
        if size in SIZE:
            pass
        elif size == "":
            pass
        else:
            raise PexelsError("Invalid photo size given, the supported ones are large, medium and small.")

        if locale == "":
            pass
        elif locale not in LOCALE_SUPPORTED:
            raise PexelsError("Invalid locale given.")

        if per_page > 80:
            raise PexelsError("per_page can not be more than 80")

        params = {
            'orientation': orientation, 
            'size': size, 
            'locale': locale, 
            'page': page,
            'per_page': per_page,
            **kwargs
        }

        data, req = self._make_request(f"search?query={query}", search_type="video", query=params)
        return VideoResponse(**data)

    def get_popular_videos(
        self,
        min_width: Optional[int],
        min_height: Optional[int],
        min_duration: Optional[int],
        max_duration: Optional[int],
        page: Optional[int] = 1,
        per_page: Optional[int] = 15,
        **kwargs
    ) -> VideoResponse:
        """
        This method enables you to receive the current popular Pexels videos. 

        Args:
            min_width (:obj:`int`, optional): The minimum width in pixels of the returned videos.
            min_height (:obj:`int`, optional): The maximum height in pixels of the returned videos.
            min_duration (:obj:`int`, optional): The minimum duration in seconds of the returned videos.
            max_duration (:obj:`int`, optional): The maximum duration in seconds of the returned videos.
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.VideoResponse`
         
        Raises:
            PexelsError: When `per_page` is above 80.
        """

        if per_page > 80:
            raise PexelsError("per_page can not be more than 80")

        params = {
            'min_width': min_width,
            'min_height': min_height,
            'min_duration': min_duration,
            'max_duration': max_duration,
            'page': page,
            **kwargs
        }

        data, req = self._make_request("popular", "video", query=params)
        return VideoResponse(**data)

    def get_video(self, id: int) -> Video:
        """
        Retrieve a specific `Video` from its id. 
        
        Args:
            id (:obj:`int`): The id of the video you are requesting.
        
        Returns:
            :class:`Pexels.types.Video`
        """
        
        data, req = self._make_request(f"videos/{id}", "video")
        return Video(**data)
    
    def get_featured_collections(self, page: Optional[int] = 1, per_page: Optional[int] = 15, **kwargs) -> CollectionResponse:
        """
        This method returns all featured collections on Pexels. 
        
        Args:
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.CollectionResponse`
        
        Raises:
            PexelsError: When `per_page` is above 80.
        """
        if per_page > 80:
            raise PexelsError("per_page can not be more than 80.")

        params = {'page': page, 'per_page': per_page}

        data, req = self._make_request("collections/featured", "photo", query=params)
        return CollectionResponse(**data)

    def get_my_collections(self, page: Optional[int] = 1, per_page: Optional[int] = 15, **kwargs) -> CollectionResponse:
        """
        This method returns all of your collections. 
        
        Args:
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.CollectionResponse`
        
        Raises:
            PexelsError: When `per_page` is above 80."""

        if per_page > 80:
            raise PexelsError("per_page can not be more than 80.")

        params = {'page': page, 'per_page': per_page}

        data, req = self._make_request("collections", "photo", query=params)
        return CollectionResponse(**data)
    
    def get_collection_media(self,id: str, type: Optional[str] = "", page: Optional[int] = 1, per_page: Optional[int] = 15, **kwargs) -> CollectionMediaResponse:
        """
        This method returns all featured collections on Pexels. 
        
        Args:
            type (:obj:`str`, optional): The type of media you are requesting, If not given or if given with an invalid
                value, all media will be returned. Supported values are `photos` and `videos`
            page (:obj:`int`, optional): The page number you are requesting. Default: 1
            per_page (:obj:`int`, optional): The number of results you are requesting per page. Default: 15 Max: 80
        
        Returns:
            :class:`Pexels.types.CollectionResponse`
        
        Raises:
            PexelsError: When `per_page` is above 80.
        """

        if per_page > 80:
            raise PexelsError("per_page can not be more than 80.")

        params = {"type": type, "page": page, "per_page": per_page}

        data , req = self._make_request(f"collections/{id}", "photo", query=params)
        return CollectionMediaResponse(**data)



from re import U
from typing import Any, Dict, List, Optional, Union

 
class PexelsType:
    """Base class for all pexels objects"""
 
    def __str__(self) -> str:
        return f'<{self.__class__.__name__}: {self.__dict__}'
    
    def __repr__(self) -> str:
        return self.__str__()
 
class Src(PexelsType):
 
    original: str
    "The image without any size changes. It will be the same as the width and height attributes."
    large: str
    "The image resized to W 940px X H 650px DPR 1."
    large2x: str
    "The image resized W 940px X H 650px DPR 2."
    medium: str
    "The image resized W 940px X H 650px DPR 2."
    small: str
    "The image scaled proportionally so that it's new height is 130px."
    portrait: str
    "The image cropped to W 800px X H 1200px."
    landscape: str
    "The image cropped to W 1200px X H 627px."
    tiny: str
    "The image cropped to W 280px X H 200px."
 
    def __init__(
        self,
        original: str,
        large: str,
        large2x: str,
        medium: str,
        small: str,
        portrait: str,
        landscape: str,
        tiny: str,
        **kwargs
    ) -> None:
 
        self.original = original
        self.large = large
        self.large2x = large2x
        self.medium = medium
        self.small = small
        self.portrait = portrait
        self.landscape = landscape
        self.tiny = tiny
 
class User(PexelsType):

    id: int
    "The id of the videographer."
    name: str
    "The name of the videographer."
    url: str
    "The URL of the videographer's Pexels Profile."

    def __init__(self, id: int, name: str, url: str, **kwargs):

        self.id = id
        self.name = name
        self.url = url

class Photo(PexelsType):
    
    type: str
    "The type of media to be shown in collections."
    id: int
    "The id of the photo."
    width: int
    "The real width of the photo in pixels."
    height: int
    "The real height of the photo in pixels."
    url: str
    "The Pexels URL where the photo is located."
    photographer: str
    "The name of the photographer who took the photo."
    photographer_url: str
    "The URL of the photographer's Pexels profile."
    photographer_id: str
    "The id of the photographer."
    avg_color: str
    "The average color of the photo. Useful for a placeholder while the image loads"
    src: Src
    "An assortment of different image sizes that can be used to display this Photo."
    alt: str
    "Text description of the photo for use in the alt attribute."
 
    def __init__(
        self,
        id: int,
        width: int,
        height: int,
        url: str,
        photographer: str,
        photographer_url: str,
        photographer_id: int,
        avg_color: str,
        src: Src,
        alt: str,
        type: str = "Photo",
        **kwargs
    ) -> None:
 
        self.id = id
        self.width = width
        self.height = height
        self.url = url
        self.photographer = photographer
        self.photographer_url = photographer_url
        self.photographer_id = photographer_id
        self.avg_color = avg_color
        self.src = Src(**src)
        self.alt = alt
 
class VideoFiles(PexelsType):

    id: int
    "The id of the `video_file`"
    quality: str
    "The video quality of the `video_file`"
    file_type: str
    "The video format of the `video_file`"
    width: int
    "The width of the `video_file` in pixels."
    height: int
    "The height of the `video_file` in pixels."
    link: str
    "A link to where the `video_file` is hosted."

    def __init__(
        self,
        id: int,
        quality: str,
        file_type: str,
        width: int,
        height: int,
        link:str,
        **kwargs
    ):

        self.id = id
        self.quality = quality
        self.file_type = file_type
        self.width = width
        self.height = height
        self.link = link

class VideoPicture(PexelsType):

    id: int
    "The id of the `video_picture`"
    picture: str
    "A link to the preview image."
    nr: int

    def __init__(self, id: int, picture: str, nr:int, **kwargs):

        self.id = id
        self.picture = picture
        self.nr = nr 

class PhotoResponse(PexelsType):
 
    photos = List[Photo]
    "A list of `Photo` object"
    page = int
    "The current page number"
    per_page = int
    "The number of results returned with each page"
    total_results = int
    "The total number of results for the request"
    prev_page = str
    "URL for the previous page of results, if applicable"
    next_page = str
    "URL for the next page of results, if applicable"
 
    def __init__(
        self,
        photos: List[Photo],
        page: int,
        per_page: int,
        total_results: int,
        prev_page: str = "",
        next_page: str = "",
        **kwargs
        ):
        #assign photos dict to self.photos
        self.photos = [Photo(**photo) for photo in photos]
        self.page = page
        self.per_page = per_page
        self.total_results = total_results
        self.prev_page = prev_page
        self.next_page = next_page

class Video:
    
    type: str
    "The type of this media to be shown collections."
    id: int
    "The id of the video."
    width: int
    "The real width of the video in pixels."
    height: int
    "The real height of the video in pixels."
    url: str
    "The pexels URL where the video is located."
    image: str
    "URL to a screenshot of the video."
    duration: int
    "The duration of the video in seconds."
    user: User
    "The videographer who shot the video."
    video_files: List[VideoFiles]
    "A list of different sized versions of the video."
    video_pictures: List[VideoPicture]
    "A list of preview pictures of the video."

    def __init__(
        self,
        id: int,
        width: int,
        height: int,
        url: str,
        image: str,
        duration: int,
        user: User,
        video_files: List[VideoFiles],
        video_pictures: List[VideoPicture],
        type: str = "Video",
        **kwargs
    ):
        self.type = type
        self.id = id
        self.width = width
        self.height = height
        self.url = url
        self.image = image
        self.duration = duration
        self.user = user
        self.video_files = [VideoFiles(**vid_file) for vid_file in video_files]
        self.video_pictures = [VideoPicture(**vid_pic) for vid_pic in video_pictures]

class VideoResponse(PexelsType):

    videos: List[Video]
    "A list of `Video` objects."
    url: str
    "The Pexels URL for the current search query."
    page:int
    "The current page number."
    per_page: int
    "The number of results returned with each page."
    total_results: int
    "The total number of results for the request."
    prev_page: str
    "URL for the previous page of results, if applicable."
    next_page: str
    "URL for the next page of results, if applicable."

    def __init__(
        self,
        videos: List[Video],
        url: str,
        page: int,
        per_page: int,
        total_results: int,
        prev_page: Optional[str] = "",
        next_page: Optional[str] = "",
        **kwargs
    ):

        self.videos = [Video(**video) for video in videos]
        self.url = url
        self.page = page
        self.per_page = per_page
        self.total_results = total_results
        self.prev_page = prev_page
        self.next_page = next_page

class Collection(PexelsType):

    id: str
    "The id of the collection."
    title: str
    "The name of the collection."
    description: str
    "The description of the collection."
    private: bool
    "Whether or not  the collection is marked as private."
    media_count: int
    "The total number of media included in this collection."
    photos_count: int
    "The total number of photos included in this collection."
    videos_count: int
    "The total number of videos included in this collection."

    def __init__(
        self,
        id: str,
        title: str,
        description: str,
        private: bool,
        media_count: int,
        photos_count: int,
        videos_count: int,
        **kwargs
    ):

        self.id = id
        self.title = title
        self.description = description
        self.private = private
        self.media_count = media_count
        self.photos_count = photos_count
        self.videos_count = videos_count

class CollectionResponse(PexelsType):

    collections: List[Collection]
    "A list of collection objects."
    page: int
    "The current page number."
    per_page: int
    "The number of results returned with each page."
    total_results: int
    "The total number of results for the request."
    prev_page: Optional[str]
    "URL for the previous page of results, if applicable"
    next_page: Optional[str]
    "URL for the next page of results, if applicable."

    def __init__(
        self,
        collections: List[Collection],
        page: int,
        per_page: int,
        total_results: int,
        prev_page: Optional[str] = "",
        next_page: Optional[str] = "",
        **kwargs
    ):

        self.collections = [Collection(**collection) for collection in collections]
        self.page = page
        self.per_page = per_page
        self.total_results = total_results
        self.prev_page = prev_page
        self.next_page = next_page

class CollectionMediaResponse(PexelsType):

    id: str
    "The id of the collection you are requesting."
    media: List[Union[Photo, Video]]
    "A list of media objects. Each object has an extra type attribute to indicate the type of object."
    page: int
    "The current page number."
    per_page: int
    "The number of results returned with each page."
    total_results: int
    "The total number of results for the request."
    prev_page: Optional[str]
    "URL for the previous page of results, if applicable."
    next_page: Optional[str]
    "URL for the next page of results, if applicable."

    def __init__(
        self,
        id: str,
        media: List[Union[Photo, Video]],
        page: int,
        per_page: int,
        total_results: int,
        prev_page: Optional[str] = "",
        next_page: Optional[str] = "",
        **kwargs
    ):

        self.id = id
        self.media = []
        for _media in media:
            if _media["type"] == "Photo":
                self.media.append(Photo(**_media))
            elif _media["type"] == "Video":
                self.media.append(Video(**_media))
            else:
                pass
        
        self.page = page
        self.per_page = per_page
        self.total_results = total_results
        self.prev_page = prev_page
        self.next_page = next_page


"""All constants to be used are available here"""

from typing import List

ORIENTATION: List[str] = ['landscape', 'portrait', 'square']
"Desired photo orientation."

SIZE: List[str] = ['large', 'medium', 'small']
"Minimum photo size."

COLOR: List[str] = ['red', 'orange', 'yellow', 'green', 'turquoise', 'blue', 'violet', 'pink', 'brown', 'black', 'gray', 'white']
"Desired Photo color."

LOCALE_SUPPORTED: List[str] = [
    'en-US',
    'pt-BR',
    'es-ES',
    'ca-ES',
    'de-DE',
    'it-IT',
    'fr-FR',
    'sv-SE',
    'id-ID',
    'pl-PL', 
    'ja-JP',
    'zh-TW',
    'zh-CN',
    'ko-KR',
    'th-TH',
    'nl-NL',
    'hu-HU',
    'vi-VN',
    'cs-CZ',
    'da-DK',
    'fi-FI',
    'uk-UA',
    'el-GR',
    'ro-RO',
    'nb-NO',
    'sk-SK',
    'tr-TR',
    'ru-RU'
]
"The locale of the search you are performing."


"""A unofficial python wrapper library for Pexels API"""

from .client import Client
from .errors import APIError, InvalidTokenError, QuotaExceedError
from .errors import PexelsError, APIError, QuotaExceedError, InvalidTokenError
from .constants import COLOR, ORIENTATION, SIZE, LOCALE_SUPPORTED

__version__ = "1.1"

__author__ = "jokerhacker.6521@protonmail.com"
